"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Auth = void 0;
const command_1 = require("@oclif/command");
const chalk = require("chalk");
const inquirer = require("inquirer");
const axios_1 = require("axios");
const express = require("express");
const open = require("open");
const TokenConfig_1 = require("../utils/TokenConfig");
const Config_1 = require("../utils/Config");
const figchalk = require("figchalk");
const tokenConfig = new TokenConfig_1.default();
const configData = Config_1.getConfig();
class Auth extends command_1.default {
    async init() { }
    async authorize() {
        inquirer
            .prompt({
            name: "authorize",
            type: "confirm",
            message: "Would you like us to open browser for authentication?",
            default: true,
        })
            .then(async (answer) => {
            if (answer.authorize) {
                // open browser and authorize!
                try {
                    const app = express();
                    app.use(express.json());
                    app.use(express.urlencoded({ extended: false }));
                    const server = await app.listen(3000);
                    let resolve;
                    const p = new Promise((_resolve) => {
                        resolve = _resolve;
                    });
                    app.get("/callback", (req, res) => {
                        resolve(req.query.code);
                        res.redirect("done");
                    });
                    app.get("/done", (req, res) => {
                        res.send("Successfully authorized!<br/>Now you can close this browser and return to CLI!");
                    });
                    this.log(configData.issuerBaseURL);
                    open(`${configData.issuerBaseURL}/authorize?response_type=code&client_id=${configData.clientID}&redirect_uri=${configData.baseURL}/callback&scope=openid%20profile%20email&state=testing`);
                    // Wait for the first auth code
                    const code = await p;
                    const response = await axios_1.default.post(`${configData.issuerBaseURL}/oauth/token`, `grant_type=authorization_code&client_id=${configData.clientID}&client_secret=${configData.clientSecret}&code=${code}&redirect_uri=${configData.baseURL}/callback`);
                    const access_token = response.data["access_token"];
                    await tokenConfig.setToken(access_token, this.config.windows);
                    server.close();
                    const user = await tokenConfig.getUser(access_token);
                    this.log("Logged in successfully as " + chalk.greenBright(user.email));
                    process.exit(0);
                }
                catch (error) {
                    this.log(error.response);
                }
            }
            else {
                this.log(chalk.yellow("\nIf you're not authorized,\nYou won't be able to use Noteli."));
            }
        })
            .catch((err) => this.log(err));
    }
    async run() {
        console.log(figchalk.mix("N o t e l i", "redBright"));
        tokenConfig
            .getToken(this.config.windows)
            .then(async (token) => {
            if (token == "") {
                await this.authorize();
            }
            else {
                tokenConfig
                    .getUser(token)
                    .then(async (user) => {
                    if (user != null) {
                        this.log(chalk.yellow("An account already exists!"));
                        this.log("Logged in as " + chalk.greenBright(user.email));
                    }
                    else {
                        await this.authorize();
                    }
                })
                    .catch(async (err) => {
                    await this.authorize();
                });
            }
        })
            .catch(async (err) => {
            await this.authorize();
        });
    }
}
exports.Auth = Auth;
